// node_modules/.pnpm/nanoid@5.0.6/node_modules/nanoid/non-secure/index.js
var urlAlphabet = "useandom-26T198340PX75pxJACKVERYMINDBUSHWOLF_GQZbfghjklqvwyzrict";
var nanoid = (size = 21) => {
  let id = "";
  let i = size;
  while (i--) {
    id += urlAlphabet[Math.random() * 64 | 0];
  }
  return id;
};

// src/connection.ts
var ICE_SERVERS = [
  {
    urls: "stun:stun.l.google.com:19302"
  }
];
var MCHEConnection = class {
  #peerConnection = new RTCPeerConnection({
    iceServers: ICE_SERVERS
  });
  #datachannel = null;
  #datachannelCallbacks = [];
  constructor(onDataChannelMessage) {
    this.#peerConnection.ondatachannel = (event) => {
      this.#datachannel = event.channel;
      this.#datachannelCallbacks.forEach((callback) => {
        callback(event.channel);
      });
    };
    this.#datachannelCallbacks.push((channel) => {
      channel.onmessage = (event) => {
        onDataChannelMessage(event);
      };
    });
  }
  async createOffer() {
    this.#datachannel = this.#peerConnection.createDataChannel("datachannel");
    const offer = await this.#peerConnection.createOffer();
    await this.#peerConnection.setLocalDescription(offer);
    return offer;
  }
  async receiveOffer(offer) {
    await this.#peerConnection.setRemoteDescription(offer);
  }
  async createAnswer() {
    const answer = await this.#peerConnection.createAnswer();
    await this.#peerConnection.setLocalDescription(answer);
    return answer;
  }
  async receiveAnswer(answer) {
    await this.#peerConnection.setRemoteDescription(answer);
  }
  async addIceCandidate(candidate) {
    await this.#peerConnection.addIceCandidate(candidate);
  }
  async createDataChannel(label) {
    const channel = this.#peerConnection.createDataChannel(label);
    return channel;
  }
  onCandidate(callback) {
    this.#peerConnection.onicecandidate = (event) => {
      if (event.candidate)
        callback(event.candidate);
    };
  }
  addDataChannelCallback(callback) {
    if (this.#datachannel) {
      callback(this.#datachannel);
      return;
    }
    this.#datachannelCallbacks.push(callback);
  }
};

// src/signalling.ts
import { createWSHE } from "wshe";
var SignallingServerClient = class {
  #ws;
  constructor(url) {
    this.#ws = createWSHE(url, {
      immediate: true
    });
  }
  send(topic, data) {
    this.#ws.send(topic, data);
  }
  async wait(topic) {
    return new Promise((resolve) => {
      const fn = (data) => {
        resolve(data);
      };
      this.#ws.subscribe(topic, fn);
    });
  }
};

// src/utils.ts
function error(error2) {
  console.error(`[mche Error] ${error2.message}`);
  console.error(error2.stack);
}
function log(...args) {
  console.log("[mche Log]", ...args);
}

// src/index.ts
var randomId = () => nanoid(6);
var MCHelper = class {
  #options;
  #id;
  #connectionPool = /* @__PURE__ */ new Map();
  #signallingServer;
  #dataBuffer = [];
  #dataBufferCallbacks = [];
  constructor(options) {
    this.#options = options;
    this.#id = options.id || randomId();
    this.#signallingServer = new SignallingServerClient(options.signallingServerUrl);
    this.#bindEvents().catch(error);
    this.#signallingServer.wait("join").then((id) => {
      this.registerRemote(id);
    }).catch(error);
  }
  async #bindEvents() {
    this.#signallingServer.wait("mche:offer:receive" /* OfferReceive */).then(async ({ receiver, offer, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received offer from", sender, "payload is", offer);
      let connection = this.#connectionPool.get(sender);
      if (!connection) {
        connection = new MCHEConnection(this.#addMessage.bind(this));
        this.#connectionPool.set(sender, connection);
      }
      await connection.receiveOffer(offer);
      const answer = await connection.createAnswer();
      const payload = {
        answer,
        // swap sender and receiver
        receiver: sender,
        sender: receiver
      };
      if (this.#options.debug)
        log("Sending answer to", sender, "payload is", payload);
      this.#signallingServer.send("mche:answer:send" /* AnswerSend */, payload);
    }).catch(error);
    this.#signallingServer.wait("mche:answer:receive" /* AnswerReceive */).then(async ({ receiver, answer, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received answer from", sender, "payload is", answer);
      const connection = this.#connectionPool.get(sender);
      if (!connection)
        return;
      await connection.receiveAnswer(answer);
    }).catch(error);
    this.#signallingServer.wait("mche:candidate:receive" /* CandidateReceive */).then(async ({ receiver, candidate, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received candidate from", sender, "payload is", candidate);
      const connection = this.#connectionPool.get(sender);
      if (!connection)
        return;
      await connection.addIceCandidate(candidate);
    }).catch(error);
  }
  #addMessage(message) {
    this.#dataBuffer.push(message);
    this.#dataBufferCallbacks.forEach((callback) => {
      callback(message);
      this.#dataBuffer.length = 0;
    });
  }
  /**
   * You should call this function to connection other peers.
   * @returns The ID of the peer. You need **hold** this ID to do next operations.
   */
  async registerRemote(remoteId) {
    const nonNullRemoteId = remoteId || randomId();
    const connection = new MCHEConnection(this.#addMessage.bind(this));
    connection.onCandidate((candidate) => {
      const payload = {
        receiver: nonNullRemoteId,
        sender: this.#id,
        candidate
      };
      if (this.#options.debug)
        log("Sending candidate to", nonNullRemoteId, "payload is", payload);
      this.#signallingServer.send("mche:candidate:send" /* CandidateSend */, payload);
    });
    this.#connectionPool.set(nonNullRemoteId, connection);
    await this.#connect(nonNullRemoteId);
    return nonNullRemoteId;
  }
  async #connect(peerId) {
    const connection = this.#connectionPool.get(peerId);
    if (!connection) {
      error(new Error(`Peer ${peerId} not found.`));
      return;
    }
    const offer = await connection.createOffer();
    const payload = {
      offer,
      sender: this.#id,
      receiver: peerId
    };
    if (this.#options.debug)
      log("Sending offer to", peerId, "payload is", payload);
    this.#signallingServer.send("mche:offer:send" /* OfferSend */, payload);
  }
  broadcast(message) {
    const payload = {
      sender: this.#id,
      receiver: "broadcast",
      message
    };
    if (this.#options.debug)
      log("Broadcasting message", message, "payload is", payload);
    this.#connectionPool.forEach((connection) => {
      connection.addDataChannelCallback((channel) => {
        channel.send(JSON.stringify(payload.message));
      });
    });
  }
  onBroadcast(callback) {
    this.#dataBufferCallbacks.push((message) => {
      callback(message);
    });
  }
};
export {
  MCHelper
};
